@@ -44,7 +44,8 @@ from health_azure.utils import (ENV_EXPERIMENT_NAME, create_python_environment,
                                 create_python_environment_v2, register_environment_v2, V2_INPUT_DATASET_PATTERN,
                                 V2_OUTPUT_DATASET_PATTERN, wait_for_job_completion)
 from health_azure.datasets import (DatasetConfig, StrOrDatasetConfig, setup_local_datasets,
-                                   _input_dataset_key, _output_dataset_key, _replace_string_datasets)
+                                   _input_dataset_key, _output_dataset_key, _replace_string_datasets,
+                                   _get_or_create_v2_data_asset)
 
 
 logger = logging.getLogger('health_azure')
@@ -415,22 +416,6 @@ def _generate_output_dataset_command(output_datasets_v2: Dict[str, Output]) -> s
     return output_cmd
 
 
-def get_display_name_v2(tags: Optional[Dict[str, Any]] = None) -> str:
-    """
-    If the command line argument 'tag' is provided, return its value to be set as the job's display name.
-    Empty spaces in the tag will be replaced with hyphens, otherwise AML treats it as multiple statements.
-    Otherwise return an empty string.
-
-    :param tags: An optional dictionary of tag names and values to be provided to the job.
-    :return: A string either containing the value of tag, or else empty.
-    """
-    if tags is None:
-        return ""
-    tag = tags.get("tag", "")
-    display_name = tag.replace(" ", "-")
-    return display_name
-
-
 def effective_experiment_name(experiment_name: Optional[str],
                               entry_script: Optional[PathOrString] = None) -> str:
     """Choose the experiment name to use for the run. If provided in the environment variable HIML_EXPERIMENT_NAME,
@@ -472,6 +457,7 @@ def submit_run_v2(workspace: Optional[Workspace],
                   hyperparam_args: Optional[Dict[str, Any]] = None,
                   num_nodes: int = 1,
                   pytorch_processes_per_node: Optional[int] = None,
+                  display_name: Optional[str] = None,
                   ) -> Job:
     """
     Starts a v2 AML Job on a given workspace by submitting a command
@@ -501,6 +487,8 @@ def submit_run_v2(workspace: Optional[Workspace],
     :param pytorch_processes_per_node: For plain PyTorch multi-GPU processing: The number of processes per node.
         If supplied, it will run a command job with the "pytorch" framework (rather than "Python"), and using "nccl"
         as the communication backend.
+    :param display_name: The name for the run that will be displayed in the AML UI. If not provided, a random
+        display name will be generated by AzureML.
     :return: An AzureML Run object.
     """
     if ml_client is None:
@@ -535,7 +523,6 @@ def submit_run_v2(workspace: Optional[Workspace],
         output_datasets_v2 = {}
 
     job_to_submit: Union[Command, Sweep]
-    display_name = get_display_name_v2(tags)
 
     # number of nodes and processes per node cannot be less than one
     if num_nodes < 1:
@@ -634,6 +621,7 @@ def submit_run(workspace: Workspace,
                tags: Optional[Dict[str, str]] = None,
                wait_for_completion: bool = False,
                wait_for_completion_show_output: bool = False,
+               display_name: Optional[str] = None,
                ) -> Run:
     """
     Starts an AzureML run on a given workspace, via the script_run_config.
@@ -648,6 +636,8 @@ def submit_run(workspace: Workspace,
         the completion of this run (if True).
     :param wait_for_completion_show_output: If wait_for_completion is True this parameter indicates whether to show the
         run output on sys.stdout.
+    :param display_name: The name for the run that will be displayed in the AML UI. If not provided, a random
+        display name will be generated by AzureML.
     :return: An AzureML Run object.
     """
     cleaned_experiment_name = to_azure_friendly_string(experiment_name)
@@ -665,6 +655,8 @@ def submit_run(workspace: Workspace,
             # It is probably a HyperDriveConfig
             tags = {"commandline_args": " ".join(script_run_config.run_config.arguments)}
     run.set_tags(tags)
+    if display_name:
+        run.display_name = display_name
 
     _write_run_recovery_file(run)
 
@@ -704,8 +696,17 @@ def create_v2_inputs(ml_client: MLClient, input_datasets: List[DatasetConfig]) -
     inputs: Dict[str, Input] = {}
     for i, input_dataset in enumerate(input_datasets):
         input_name = f"INPUT_{i}"
-        version = input_dataset.version or 1
-        data_asset: Data = ml_client.data.get(input_dataset.name, version=str(version))
+        version = input_dataset.version
+        # if version is None, this function gets the latest version
+        logging.info(
+            f"Trying to access data asset {input_dataset.name} version {version}, datastore {input_dataset.datastore}"
+        )
+        data_asset: Data = _get_or_create_v2_data_asset(
+            ml_client,
+            input_dataset.datastore,
+            input_dataset.name,
+            version=str(version) if version else None,
+        )
         if not data_asset.path:
             raise ValueError(f"Data asset {data_asset.id} has no path.")
         # Some mismatches with the documentation here:
@@ -778,6 +779,7 @@ def submit_to_azure_if_needed(  # type: ignore
         hyperparam_args: Optional[Dict[str, Any]] = None,
         strictly_aml_v1: bool = False,
         pytorch_processes_per_node_v2: Optional[int] = None,
+        display_name: Optional[str] = None,
 ) -> AzureRunInfo:  # pragma: no cover
     """
     Submit a folder to Azure, if needed and run it.
@@ -845,6 +847,8 @@ def submit_to_azure_if_needed(  # type: ignore
     :param pytorch_processes_per_node_v2: For plain PyTorch multi-GPU processing: The number of processes per node. This
         is only supported with AML SDK v2, and ignored in v1. If supplied, the job will be submitted as using the
         "pytorch" framework (rather than "Python"), and using "nccl" as the communication backend.
+    :param display_name: The name for the run that will be displayed in the AML UI. If not provided, a random
+        display name will be generated by AzureML.
     :return: If the script is submitted to AzureML then we terminate python as the script should be executed in AzureML,
         otherwise we return a AzureRunInfo object.
     """
@@ -955,6 +959,7 @@ def submit_to_azure_if_needed(  # type: ignore
                              experiment_name=effective_experiment_name(experiment_name, script_run_config.script),
                              script_run_config=config_to_submit,
                              tags=tags,
+                             display_name=display_name,
                              wait_for_completion=wait_for_completion,
                              wait_for_completion_show_output=wait_for_completion_show_output)
             if after_submission is not None:
@@ -985,6 +990,7 @@ def submit_to_azure_if_needed(  # type: ignore
                                 script_params=script_params,
                                 compute_target=compute_cluster_name,
                                 tags=tags,
+                                display_name=display_name,
                                 docker_shm_size=docker_shm_size,
                                 wait_for_completion=wait_for_completion,
                                 hyperparam_args=hyperparam_args,
